{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Menus;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, VCL.Controls, VCL.Forms, VCL.StdCtrls, SysUtils, VCL.Graphics, Types,
  Vcl.Menus, WEBLib.Controls, WEBLib.StdCtrls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Controls, Forms, StdCtrls, SysUtils, Graphics, Types,
  Menus, WEBLib.Controls, WEBLib.StdCtrls;
  {$ENDIF}

type
  TWebCustomMainMenu = class(TMainMenu)
  end;

  TMainMenuHamburgerMenuVisible = (hmAlways, hmNever, hmResponsive);

  TMainMenuAppearance = class;

  THamburgerMenu = class(TPersistent)
  private
    FOwner: TMainMenuAppearance;
    FVisible: TMainMenuHamburgerMenuVisible;
    FCaption: string;
    FBackgroundColor: TColor;
    FCaptionColor: TColor;
    FResponsiveMaxWidth: Integer;
  public
    constructor Create(AOwner: TMainMenuAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Caption: string read FCaption write FCaption;
    property CaptionColor: TColor read FCaptionColor write FCaptionColor default clWhite;
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default clSilver;
    property Visible: TMainMenuHamburgerMenuVisible read FVisible write FVisible default hmResponsive;
    property ResponsiveMaxWidth: Integer read FResponsiveMaxWidth write FResponsiveMaxWidth default 768;
  end;

  TMainMenuAppearance = class(TPersistent)
  private
    FOwner: TMainMenu;
    FBackgroundColor: TColor;
    FHoverFontColor: TColor;
    FHoverColor: TColor;
    FImageURLs: TStringList;
    FHamburgerMenu: THamburgerMenu;
    FImageSize: Integer;
    FSubmenuIndicator: string;
    procedure SetImageURLs(const Value: TStringList);
  public
    constructor Create(AOwner: TMainMenu); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default $F0F0F0;
    property HamburgerMenu: THamburgerMenu read FHamburgerMenu write FHamburgerMenu;
    property HoverColor: TColor read FHoverColor write FHoverColor default clHighlight;
    property HoverFontColor: TColor read FHoverFontColor write FHoverFontColor default clHighlightText;
    property ImageURLs: TStringList read FImageURLs write SetImageURLs;
    property ImageSize: Integer read FImageSize write FImageSize default 16;
    property SubmenuIndicator: string read FSubmenuIndicator write FSubmenuIndicator;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebMainMenu = class(TWebCustomMainMenu)
  private
    FVisible: Boolean;
    FAppearance: TMainMenuAppearance;
    FElementClassName: TElementClassName;
    FContainer: TControl;
    FParent: TCustomControl;
    FWidth: integer;
    FHeight: integer;
    FEnabled: boolean;
    FShowHint: boolean;
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BeginUpdate;
    procedure EndUpdate;
    property Parent: TCustomControl read FParent write FParent;
  published
    property Appearance: TMainMenuAppearance read FAppearance write FAppearance;
    property Container: TControl read FContainer write FContainer;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property Enabled: boolean read FEnabled write FEnabled default true;
    property ShowHint: boolean read FShowHint write FShowHint default false;
    property Visible: Boolean read FVisible write FVisible default True;
    property Width: integer read FWidth write FWidth default 0;
    property Height: integer read FHeight write FHeight default 0;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebPopupMenu = class(TPopupMenu)
  end;

implementation

{ TWebMainMenu }

procedure TWebMainMenu.BeginUpdate;
begin
  //
end;

constructor TWebMainMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAppearance := TMainMenuAppearance.Create(Self);
  FContainer := nil;
  FElementClassName := '';
  FVisible := True;
  FShowHint := False;
  FEnabled := True;
  FWidth := 0;
  FHeight := 0;
end;

destructor TWebMainMenu.Destroy;
begin
  FAppearance.Free;
  inherited;
end;

procedure TWebMainMenu.EndUpdate;
begin
  //
end;

procedure TWebMainMenu.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = Container) then
    Container := nil;
end;

{ TMainMenuAppearance }

procedure TMainMenuAppearance.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TMainMenu) then
  begin
    FBackgroundColor := (Source as TMainMenuAppearance).BackgroundColor;
    FHoverColor := (Source as TMainMenuAppearance).HoverColor;
    FHoverFontColor := (Source as TMainMenuAppearance).HoverFontColor;
    FHamburgerMenu.Assign((Source as TMainMenuAppearance).HamburgerMenu);
    FImageURLs.Assign((Source as TMainMenuAppearance).ImageURLs);
    FImageSize := (Source as TMainMenuAppearance).ImageSize;
    FSubmenuIndicator := (Source as TMainMenuAppearance).SubmenuIndicator;
  end;
end;

constructor TMainMenuAppearance.Create(AOwner: TMainMenu);
begin
  FBackgroundColor := $F0F0F0;
  FHoverColor := clHighlight;
  FHoverFontColor := clHighlightText;
  FHamburgerMenu := THamburgerMenu.Create(Self);
  FImageURLs := TStringList.Create;
  FImageSize := 16;
  FSubmenuIndicator := '&#9658;';
  FOwner := AOwner;
end;

destructor TMainMenuAppearance.Destroy;
begin
  FHamburgerMenu.Free;
  FImageURLs.Free;
  inherited;
end;

function TMainMenuAppearance.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


procedure TMainMenuAppearance.SetImageURLs(const Value: TStringList);
begin
  FImageURLs.Assign(Value);
end;

{ THamburgerMenu }

procedure THamburgerMenu.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is THamburgerMenu) then
  begin
    FBackgroundColor := (Source as THamburgerMenu).BackgroundColor;
    FCaption := (Source as THamburgerMenu).Caption;
    FCaptionColor := (Source as THamburgerMenu).CaptionColor;
    FVisible := (Source as THamburgerMenu).Visible;
    FResponsiveMaxWidth := (Source as THamburgerMenu).ResponsiveMaxWidth;
  end;
end;

constructor THamburgerMenu.Create(AOwner: TMainMenuAppearance);
begin
  FBackgroundColor := clSilver;
  FCaption := 'Menu';
  FCaptionColor := clWhite;
  FVisible := hmResponsive;
  FResponsiveMaxWidth := 768;
  FOwner := AOwner;
end;

destructor THamburgerMenu.Destroy;
begin
  inherited;
end;

function THamburgerMenu.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

end.
