unit WEBLib.SignatureCapture;

interface

uses
  Classes, WEBLib.Controls, Types, Contnrs, Vcl.Graphics, Vcl.Forms, Winapi.Windows,
  WEBLib.Graphics, web;

type
  TSignatureCaptureLineTextPosition = (ltpBottomLeft, ltpBottomCenter, ltpBottomRight, ltpTopLeft, ltpTopCenter, ltpTopRight, ltpXY);
  TSignatureCaptureClearButtonPosition = (cbpTopLeft, cbpTopRight, cbpBottomLeft, cbpBottomRight, cbpXY);

  TSignatureCaptureSignTextPosition = class(TPersistent)
  private
    FLineVisible: Boolean;
    FOnChange: TNotifyEvent;
    FPosition: TSignatureCaptureLineTextPosition;
    FX: Integer;
    FY: Integer;
    FLine: TPen;
    procedure SetLineVisible(const Value: Boolean);
    procedure SetPosition(const Value: TSignatureCaptureLineTextPosition);
    procedure SetX(const Value: Integer);
    procedure SetY(const Value: Integer);
    procedure SetLine(const Value: TPen);
  protected
    procedure Changed; virtual;
  public
    constructor Create;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Line: TPen read FLine write SetLine;
    property LineVisible: Boolean read FLineVisible write SetLineVisible default True;
    property Position: TSignatureCaptureLineTextPosition read FPosition write SetPosition default ltpBottomCenter;
    property X: Integer read FX write SetX default -1;
    property Y: Integer read FY write SetY default -1;
  end;

  TSignatureCapturePoint = class(TObject)
  public
    X, Y, PenWidth: Integer;
    DrawLine: Boolean;
    PenColor: TColor;
    PenStyle: TPenStyle;
  end;

  TSignatureCapturePoints = class(TObjectList)
  private
    function GetItem(Index: Integer): TSignatureCapturePoint;
    procedure SetItem(Index: Integer; const Value: TSignatureCapturePoint);
  public
    property Items[Index: Integer]: TSignatureCapturePoint read GetItem write SetItem; default;
  end;

  TSignatureCaptureClearButton = class(TPersistent)
  private
    FImage: TPicture;
    FX: Integer;
    FY: Integer;
    FPosition: TSignatureCaptureClearButtonPosition;
    FOnChange: TNotifyEvent;
    procedure SetPosition(const Value: TSignatureCaptureClearButtonPosition);
    procedure SetX(const Value: Integer);
    procedure SetY(const Value: Integer);
    procedure SetImage(const Value: TPicture);
  protected
    procedure Changed; virtual;
    procedure ImageChange(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Image: TPicture read FImage write SetImage;
    property Position: TSignatureCaptureClearButtonPosition read FPosition write SetPosition default cbpBottomLeft;
    property X: Integer read FX write SetX default -1;
    property Y: Integer read FY write SetY default -1;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSignatureCapture = class(TWebCustomControl)
  private
    FClearButton: TSignatureCaptureClearButton;
    FPen: TPen;
    FTextPosition: TSignatureCaptureSignTextPosition;
    FText: string;
    FEmpty: Boolean;
    function GetLineY: Integer;
    procedure SetText(const Value: string);
    function IsTextStored: Boolean;
  protected
    procedure Paint; override;
    procedure PaintClearButton; virtual;
    procedure PaintLine; virtual;
    procedure PaintText; virtual;
    procedure ImageChanged(Sender: TObject);
    procedure ButtonChanged(Sender: TObject);
    procedure TextPositionChanged(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetAsBase64Image: string;
    property Empty: Boolean read FEmpty write FEmpty;
  published
    property Align;
    property Anchors;
    property ChildOrder;
    property ShowHint;
    property Visible;
    property TabOrder;
    property TabStop;
    property ClearButton: TSignatureCaptureClearButton read FClearButton write FClearButton;
    property TextPosition: TSignatureCaptureSignTextPosition read FTextPosition write FTextPosition;
    property Text: string read FText write SetText stored IsTextStored;
    property Pen: TPen read FPen write FPen;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

const
  TEXTSPACING = 5;
  IMGWIDTH = 20;

{ TSignatureCapture }

procedure TWebSignatureCapture.ButtonChanged(Sender: TObject);
begin
  Invalidate;
end;

constructor TWebSignatureCapture.Create(AOwner: TComponent);
begin
  inherited;
  Width := 400;
  Height := 300;
  FText := 'Sign here';
  FClearButton := TSignatureCaptureClearButton.Create;
  FClearButton.OnChange := ButtonChanged;
  FTextPosition := TSignatureCaptureSignTextPosition.Create;
  FTextPosition.OnChange := TextPositionChanged;
  FPen := TPen.Create;
  BorderStyle := bsSingle;
end;

destructor TWebSignatureCapture.Destroy;
begin
  FClearButton.Free;
  FTextPosition.Free;
  FPen.Free;
  inherited;
end;

function TWebSignatureCapture.GetAsBase64Image: string;
begin
  Result := '';
end;

function TWebSignatureCapture.GetLineY: Integer;
begin
  Result := Round(Canvas.TextHeight(Text)) + 2 * TEXTSPACING;
end;

procedure TWebSignatureCapture.ImageChanged(Sender: TObject);
begin
  Invalidate;
end;

function TWebSignatureCapture.IsTextStored: Boolean;
begin
  Result := FText <> 'Sign here';
end;

procedure TWebSignatureCapture.Paint;
begin
  inherited;
  PaintLine;
  PaintText;
  PaintClearButton;
end;

procedure TWebSignatureCapture.PaintClearButton;
var
  h: Integer;
begin
  if Assigned(FClearButton.Image) and Assigned(FClearButton.Image.Graphic) and not FClearButton.Image.Graphic.Empty then
  begin
    case FClearButton.Position of
      cbpTopLeft:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight, ltpXY:
          begin
            Canvas.Draw(TEXTSPACING, TEXTSPACING, FClearButton.Image.Graphic);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight:
          begin
            h := GetLineY;
            Canvas.Draw(TEXTSPACING, h + TEXTSPACING, FClearButton.Image.Graphic);
          end;
        end;
      end;
      cbpTopRight:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight, ltpXY:
          begin
            Canvas.Draw(Width - FClearButton.Image.Width - TEXTSPACING, TEXTSPACING, FClearButton.Image.Graphic);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight:
          begin
            h := GetLineY;
            Canvas.Draw(Width - FClearButton.Image.Width - TEXTSPACING, h + TEXTSPACING, FClearButton.Image.Graphic);
          end;
        end;
      end;
      cbpBottomLeft:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight:
          begin
            h := GetLineY;
            Canvas.Draw(TEXTSPACING, Height - FClearButton.Image.Height - h - TEXTSPACING, FClearButton.Image.Graphic);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight, ltpXY:
          begin
            Canvas.Draw(TEXTSPACING, Height - FClearButton.Image.Height - TEXTSPACING, FClearButton.Image.Graphic);
          end;
        end;
      end;
      cbpBottomRight:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight:
          begin
            h := GetLineY;
            Canvas.Draw(Width - FClearButton.Image.Width - TEXTSPACING, Height - FClearButton.Image.Height - h - TEXTSPACING, FClearButton.Image.Graphic);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight, ltpXY:
          begin
            Canvas.Draw(Width - FClearButton.Image.Width - TEXTSPACING, Height - FClearButton.Image.Height - TEXTSPACING, FClearButton.Image.Graphic);
          end;
        end;
      end;
      cbpXY: Canvas.Draw(FClearButton.X, FClearButton.Y, FClearButton.Image.Graphic);
    end;
  end
  else
  begin
    Canvas.Pen.Color := clBlack;
    Canvas.Pen.Width := 2;
    case FClearButton.Position of
      cbpTopLeft:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight, ltpXY:
          begin
            Canvas.MoveTo(TEXTSPACING, TEXTSPACING);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, TEXTSPACING + IMGWIDTH);
            Canvas.MoveTo(TEXTSPACING, TEXTSPACING + IMGWIDTH);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, TEXTSPACING);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight:
          begin
            h := GetLineY;
            Canvas.MoveTo(TEXTSPACING, TEXTSPACING + h);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, TEXTSPACING + IMGWIDTH + h);
            Canvas.MoveTo(TEXTSPACING, TEXTSPACING + IMGWIDTH + h);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, TEXTSPACING + h);
          end;
        end;
      end;
      cbpTopRight:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight, ltpXY:
          begin
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, TEXTSPACING);
            Canvas.LineTo(Width - TEXTSPACING, TEXTSPACING + IMGWIDTH);
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, TEXTSPACING + IMGWIDTH);
            Canvas.LineTo(Width - TEXTSPACING, TEXTSPACING);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight:
          begin
            h := GetLineY;
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, TEXTSPACING + h);
            Canvas.LineTo(Width - TEXTSPACING, TEXTSPACING + IMGWIDTH + h);
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, TEXTSPACING + IMGWIDTH + h);
            Canvas.LineTo(Width - TEXTSPACING, TEXTSPACING + h);
          end;
        end;
      end;
      cbpBottomLeft:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight:
          begin
            h := GetLineY;
            Canvas.MoveTo(TEXTSPACING, Height - IMGWIDTH - TEXTSPACING - h);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, Height - TEXTSPACING - h);
            Canvas.MoveTo(TEXTSPACING, Height - TEXTSPACING - h);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, Height - IMGWIDTH - TEXTSPACING - h);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight, ltpXY:
          begin
            Canvas.MoveTo(TEXTSPACING, Height - IMGWIDTH - TEXTSPACING);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, Height - TEXTSPACING);
            Canvas.MoveTo(TEXTSPACING, Height - TEXTSPACING);
            Canvas.LineTo(TEXTSPACING + IMGWIDTH, Height - IMGWIDTH - TEXTSPACING);
          end;
        end;
      end;
      cbpBottomRight:
      begin
        case FTextPosition.Position of
          ltpBottomLeft, ltpBottomCenter, ltpBottomRight:
          begin
            h := GetLineY;
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, Height - IMGWIDTH - TEXTSPACING - h);
            Canvas.LineTo(Width - TEXTSPACING, Height - TEXTSPACING - h);
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, Height - TEXTSPACING - h);
            Canvas.LineTo(Width - TEXTSPACING, Height - IMGWIDTH - TEXTSPACING - h);
          end;
          ltpTopLeft, ltpTopCenter, ltpTopRight, ltpXY:
          begin
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, Height - IMGWIDTH - TEXTSPACING);
            Canvas.LineTo(Width - TEXTSPACING, Height - TEXTSPACING);
            Canvas.MoveTo(Width - IMGWIDTH - TEXTSPACING, Height - TEXTSPACING);
            Canvas.LineTo(Width - TEXTSPACING, Height - IMGWIDTH - TEXTSPACING);
          end;
        end;
      end;
      cbpXY:
      begin
        Canvas.MoveTo(FClearButton.X, FClearButton.Y);
        Canvas.LineTo(FClearButton.X + IMGWIDTH, FClearButton.Y + IMGWIDTH);
        Canvas.MoveTo(FClearButton.X, FClearButton.Y + IMGWIDTH);
        Canvas.LineTo(FClearButton.X + IMGWIDTH, FClearButton.Y);
      end;
    end;
  end;
end;

procedure TWebSignatureCapture.PaintLine;
var
  h: Integer;
begin
  if FTextPosition.LineVisible then
  begin
    h := GetLineY;
    Canvas.Pen.Assign(FTextPosition.Line);
    case FTextPosition.Position of
      ltpBottomLeft, ltpBottomCenter, ltpBottomRight:
      begin
        Canvas.MoveTo(0, Height - h);
        Canvas.LineTo(Width, Height - h);
      end;
      ltpTopLeft, ltpTopCenter, ltpTopRight:
      begin
        Canvas.MoveTo(0, h);
        Canvas.LineTo(Width, h);
      end;
    end;
  end;
end;

procedure TWebSignatureCapture.PaintText;
var
  r: TRect;
  tw, th: Single;
begin
  tw := Canvas.TextWidth(Text);
  th := Canvas.TextHeight(Text);
  case FTextPosition.Position of
    ltpBottomLeft: r := Rect(TEXTSPACING, Round(Height - th - TEXTSPACING), Round(TEXTSPACING + tw), Height);
    ltpBottomCenter: r := Rect(Round((Width - tw) / 2), Round(Height - th - TEXTSPACING), Round((Width + tw) / 2), Height);
    ltpBottomRight: r := Rect(Round((Width - tw - TEXTSPACING)), Round(Height - th - TEXTSPACING), Width - TEXTSPACING, Height);
    ltpTopLeft: r := Rect(TEXTSPACING, TEXTSPACING, Round(TEXTSPACING + tw), Round(TEXTSPACING + th));
    ltpTopCenter: r := Rect(Round((Width - tw) / 2), TEXTSPACING, Round((Width + tw) / 2), Round(TEXTSPACING + th));
    ltpTopRight: r := Rect(Round((Width - tw - TEXTSPACING)), TEXTSPACING, Width - TEXTSPACING, Round(TEXTSPACING + th));
    ltpXY: r := Rect(FTextPosition.X, FTextPosition.Y, Round(FTextPosition.X + tw), Round(FTextPosition.Y + th));
  end;
  DrawText(Canvas.Handle, PChar(Text), Length(Text), r, DT_TOP or DT_SINGLELINE or DT_VCENTER);
end;

procedure TWebSignatureCapture.SetText(const Value: string);
begin
  if FText <> Value then
  begin
    FText := Value;
    Invalidate;
  end;
end;

procedure TWebSignatureCapture.TextPositionChanged(Sender: TObject);
begin
  Invalidate;
end;

{ TSignatureCapturePoints }

function TSignatureCapturePoints.GetItem(
  Index: Integer): TSignatureCapturePoint;
begin
  Result := TSignatureCapturePoint(inherited Items[Index]);
end;

procedure TSignatureCapturePoints.SetItem(Index: Integer;
  const Value: TSignatureCapturePoint);
begin
  inherited Items[Index] := Value;
end;

{ TSignatureCaptureSignTextPosition }

procedure TSignatureCaptureSignTextPosition.Changed;
begin
  if Assigned(OnChange) then
    OnChange(Self);
end;

constructor TSignatureCaptureSignTextPosition.Create;
begin
  FLineVisible := True;
  FPosition := ltpBottomCenter;
  FX := -1;
  FY := -1;
  FLine := TPen.Create;
  FLine.Color := clGray;
  FLine.Width := 1;
end;

procedure TSignatureCaptureSignTextPosition.SetLine(const Value: TPen);
begin
  FLine.Assign(Value);
  Changed;
end;

procedure TSignatureCaptureSignTextPosition.SetLineVisible(const Value: Boolean);
begin
  if FLineVisible <> Value then
  begin
    FLineVisible := Value;
    Changed;
  end;
end;

procedure TSignatureCaptureSignTextPosition.SetPosition(
  const Value: TSignatureCaptureLineTextPosition);
begin
  if FPosition <> Value then
  begin
    FPosition := Value;
    Changed;
  end;
end;

procedure TSignatureCaptureSignTextPosition.SetX(const Value: Integer);
begin
  if FX <> Value then
  begin
    FX := Value;
    Changed;
  end;
end;

procedure TSignatureCaptureSignTextPosition.SetY(const Value: Integer);
begin
  if FY <> Value then
  begin
    FY := Value;
    Changed;
  end;
end;

{ TSignatureCaptureClearButton }

procedure TSignatureCaptureClearButton.Changed;
begin
  if Assigned(OnChange) then
    OnChange(Self);
end;

constructor TSignatureCaptureClearButton.Create;
begin
  FX := -1;
  FY := -1;
  FPosition := cbpBottomLeft;
  FImage := TPicture.Create;
  FImage.OnChange := ImageChange;
end;

destructor TSignatureCaptureClearButton.Destroy;
begin
  FImage.Free;
  inherited;
end;

procedure TSignatureCaptureClearButton.ImageChange(Sender: TObject);
begin
  Changed;
end;

procedure TSignatureCaptureClearButton.SetImage(const Value: TPicture);
begin
  FImage.Assign(Value);
end;

procedure TSignatureCaptureClearButton.SetPosition(
  const Value: TSignatureCaptureClearButtonPosition);
begin
  if FPosition <> Value then
  begin
    FPosition := Value;
    Changed;
  end;
end;

procedure TSignatureCaptureClearButton.SetX(const Value: Integer);
begin
  if FX <> Value then
  begin
    FX := Value;
    Changed;
  end;
end;

procedure TSignatureCaptureClearButton.SetY(const Value: Integer);
begin
  if FY <> Value then
  begin
    FY := Value;
    Changed;
  end;
end;

end.
