{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Utils;

interface

uses
  System.Types, System.SysUtils, System.DateUtils, System.Classes,
  System.IOUtils, Vcl.Controls;

const
  DayMonday = System.DateUtils.DayMonday;
  DayTuesday = System.DateUtils.DayTuesday;
  DayWednesday = System.DateUtils.DayWednesday;
  DayThursday = System.DateUtils.DayThursday;
  DayFriday = System.DateUtils.DayFriday;
  DaySaturday = System.DateUtils.DaySaturday;
  DaySunday = System.DateUtils.DaySunday;

  MonthJanuary = System.DateUtils.MonthJanuary;
  MonthFebruary = System.DateUtils.MonthFebruary;
  MonthMarch = System.DateUtils.MonthMarch;
  MonthApril = System.DateUtils.MonthApril;
  MonthMay = System.DateUtils.MonthMay;
  MonthJune = System.DateUtils.MonthJune;
  MonthJuly = System.DateUtils.MonthJuly;
  MonthAugust = System.DateUtils.MonthAugust;
  MonthSeptember = System.DateUtils.MonthSeptember;
  MonthOctober = System.DateUtils.MonthOctober;
  MonthNovember = System.DateUtils.MonthNovember;
  MonthDecember = System.DateUtils.MonthDecember;

type
  TProc = System.SysUtils.TProc;
  TProcBoolean = System.SysUtils.TProc<Boolean>;
  TProcInteger = System.SysUtils.TProc<Integer>;
  TProcString = System.SysUtils.TProc<string>;

  TCompareItems = reference to function(const ALeft, ARight: IInterface): TValueRelationship;

  IInterfaceList = System.Classes.IInterfaceList;
  IInterfaceListEx = System.Classes.IInterfaceListEx;
  IInterfaceListEx2 = interface(IInterfaceList)
    ['{4FE71218-0F8F-4027-B050-5E84E2202604}']
    procedure Sort(const ACompare: TCompareItems);
  end;
  TInterfaceList = System.Classes.TInterfaceList;

  TPath = System.IOUtils.TPath;
  TDirectory = System.IOUtils.TDirectory;

  TStringStream = System.Classes.TStringStream;
  TStringStreamHelper = class helper for TStringStream
  public
    procedure LoadFromFile(const AFileName: string; const ALoaded: TProc); overload;
    procedure LoadFromFile(const AFileName: string; const ALoaded: TProc; const AError: TProcString); overload;
  end;

  TCharHelper = record helper for Char
  public const
    Length = 1;
  public
    function IsDigit: Boolean;
    function ToOrd: Integer;
    function ToUpper: Char; overload;
    class function ToUpper(const AValue: Char): Char; overload; static;
  end;

  TIntegerHelper = record helper for Integer
  public
    function Equals(const AValue: Integer): Boolean; inline;
    function NotEquals(const AValue: Integer): Boolean; inline;
    class function Parse(const AValue: string): Integer; static;
    function ToString: string; overload;
    class function ToString(const AValue: Integer): string; overload; static;
    class function TryParse(const AString: string; out AValue: Integer): Boolean; static; inline;
  end;

  TLongIntHelper = record helper for LongInt
  public
    function Equals(const AValue: LongInt): Boolean;
    function NotEquals(const AValue: LongInt): Boolean;
    class function Parse(const AValue: string): Integer; static;
    function ToString: string; overload;
    class function ToString(const AValue: LongInt): string; overload; static;
    class function TryParse(const AString: string; out AValue: Integer): Boolean; static;
  end;

  TStringHelper = record helper for string
  strict private
    function get_Chars(const AIndex: Integer): Char; inline;
    function get_Length: Integer; inline;
    procedure set_Chars(const AIndex: Integer; const AValue: Char); inline;
    procedure set_Length(const AValue: Integer); inline;
  public const
    Empty = '';
  public
    function CompareTo(const AValue: string): Integer; overload;
    class function CompareTo(const ALeft, ARight: string): Integer; overload; static;
    function Contains(const AValue: string): Boolean;
    function Equals(const AValue: string): Boolean; inline;
    function FirstChar: Char;
    function Format(const AArgs: array of const): string; overload;
    class function Format(const AValue: string; const AArgs: array of const): string; overload; static;
    function High: Integer; overload; inline;
    function IndexOf(const AValue: Char): Integer; overload; inline;
    function IndexOf(const AValue: string; const AStartIndex, ACount: Integer): Integer; overload;
    function IndexOf(const AValue: string): Integer; overload; inline;
    function Insert(const AStartIndex: Integer; const AValue: string): string;
    function IsEmpty: Boolean; inline;
    function IsNumber: Boolean;
    class function Low: Integer; static; inline;
    function NotEquals(const AValue: string): Boolean; inline;
    function NotIsEmpty: Boolean; inline;
    function PadLeft(const ATotalWidth: Integer): string; overload; inline;
    function PadLeft(ATotalWidth: Integer; const APaddingChar: Char): string; overload; inline;
    function Remove(const AStartIndex: Integer): string; overload; inline;
    function Remove(const AStartIndex, ACount: Integer): string; overload; inline;
    function Replace(const AOldValue: string; const ANewValue: string): string; overload;
    function Same(const AValue: string): Boolean; inline;
    function Substring(const AStartIndex: Integer): string; overload; inline;
    function Substring(const AStartIndex, ALength: Integer): string; overload; inline;
    function ToLower: string;
    function ToUpper: string;
    function Trim: string;
    property Chars[const AIndex: Integer]: Char read get_Chars write set_Chars;
    property Length: Integer read get_Length write set_Length;
  end;

  TDateTimeHelper = record helper for TDateTime
  public
    function Equals(const AValue: TDateTime): Boolean; inline;
    function NotEquals(const AValue: TDateTime): Boolean; inline;
    function ToString: string; inline;
  end;

  TCaptionHelper = record helper for TCaption
  strict private
    function get_Chars(const AIndex: Integer): Char; inline;
    function get_Length: Integer; inline;
    procedure set_Chars(const AIndex: Integer; const AValue: Char); inline;
    procedure set_Length(const AValue: Integer); inline;
  public const
    Empty = '';
  public
    function CompareTo(const AValue: string): Integer; overload;
    class function CompareTo(const ALeft, ARight: string): Integer; overload; static;
    function Equals(const AValue: string): Boolean;
    function FirstChar: Char;
    function Format(const AArgs: array of const): string; overload;
    class function Format(const AValue: string; const AArgs: array of const): string; overload; static;
    function High: Integer; overload;
    function IndexOf(const AValue: Char): Integer; overload;
    function IndexOf(const AValue: string; const AStartIndex, ACount: Integer): Integer; overload;
    function IsEmpty: Boolean;
    function IsNumber: Boolean;
    class function Low: Integer; static;
    function NotEquals(const AValue: string): Boolean;
    function NotIsEmpty: Boolean;
    function PadLeft(const ATotalWidth: Integer): string; overload;
    function PadLeft(ATotalWidth: Integer; const APaddingChar: Char): string; overload;
    function Remove(const AStartIndex: Integer): string; overload;
    function Remove(const AStartIndex, ACount: Integer): string; overload;
    function Substring(const AStartIndex: Integer): string; overload;
    function Substring(const AStartIndex, ALength: Integer): string; overload;
    function ToLower: string;
    function ToUpper: string;
    property Chars[const AIndex: Integer]: Char read get_Chars write set_Chars;
    property Length: Integer read get_Length write set_Length;
  end;

  TStringBuilder = System.SysUtils.TStringBuilder;

  TMonthNameArray = array[1..12] of string;
  TMonthNames = TMonthNameArray;
  TDayNames = array[1..7] of string;
  TFormatSettings = System.SysUtils.TFormatSettings;

function DateTimeToStr(const ADateTime: TDateTime; const AFormatSettings: TFormatSettings; const AForceTimeIfZero: Boolean = False): string;
function DateToStr(const ADateTime: TDateTime; const AFormatSettings: TFormatSettings): string;
function FormatDateTime(const AFormatStr: string; const AFormatSettings: TFormatSettings; const ADateTime: TDateTime): string;
function TryStrToDate(const AString: string; out AValue: TDateTime; const AFormatSettings: TFormatSettings): Boolean; overload;

procedure UseVariable(const AValue: IInterface); overload;
procedure UseVariable(const AValue: Integer); overload;
procedure UseVariable(const AValue: string); overload;

var
  FormatSettings: TFormatSettings;

implementation

uses
  System.Math;

procedure UseVariable(const AValue: IInterface);
begin
end;

procedure UseVariable(const AValue: Integer);
begin
end;

procedure UseVariable(const AValue: string);
begin
end;

function DateTimeToStr(const ADateTime: TDateTime; const AFormatSettings: TFormatSettings; const AForceTimeIfZero: Boolean = False): string;
begin
  Result := '';
end;

function DateToStr(const ADateTime: TDateTime; const AFormatSettings: TFormatSettings): string;
begin
  Result := '';
end;

function FormatDateTime(const AFormatStr: string; const AFormatSettings: TFormatSettings; const ADateTime: TDateTime): string;
begin
  Result := '';
end;

function TryStrToDate(const AString: string; out AValue: TDateTime; const AFormatSettings: TFormatSettings): Boolean; overload;
begin
  Result := False;
end;

{ TCharHelper }

function TCharHelper.IsDigit: Boolean;
const
  cNumChars = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'];
begin
  Result := CharInSet(Self, cNumChars);
end;

function TCharHelper.ToOrd: Integer;
begin
  Result := Ord(Self);
end;

function TCharHelper.ToUpper: Char;
begin
  Result := ToUpper(Self);
end;

class function TCharHelper.ToUpper(const AValue: Char): Char;
begin
  Result := AnsiUppercase(AValue).FirstChar;
end;

{ TIntegerHelper }

function TIntegerHelper.ToString: string;
begin
  Result := ToString(Self);
end;

function TIntegerHelper.Equals(const AValue: Integer): Boolean;
begin
  Result := Self = AValue;
end;

function TIntegerHelper.NotEquals(const AValue: Integer): Boolean;
begin
  Result := not Equals(AValue);
end;

class function TIntegerHelper.Parse(const AValue: string): Integer;
begin
  Result := StrToInt(AValue);
end;

class function TIntegerHelper.ToString(const AValue: Integer): string;
begin
  Result := IntToStr(AValue);
end;

class function TIntegerHelper.TryParse(const AString: string; out AValue: Integer): Boolean;
begin
  Result := TryStrToInt(AString, AValue);
end;

{ TLongIntHelper }

function TLongIntHelper.ToString: string;
begin
  Result := ToString(Self);
end;

function TLongIntHelper.Equals(const AValue: LongInt): Boolean;
begin
  Result := Self = AValue;
end;

function TLongIntHelper.NotEquals(const AValue: LongInt): Boolean;
begin
  Result := not Equals(AValue);
end;

class function TLongIntHelper.Parse(const AValue: string): Integer;
begin
  Result := StrToInt(AValue);
end;

class function TLongIntHelper.ToString(const AValue: LongInt): string;
begin
  Result := IntToStr(AValue);
end;

class function TLongIntHelper.TryParse(const AString: string; out AValue: Integer): Boolean;
begin
  Result := TryStrToInt(AString, AValue);
end;

{ TStringHelper }

function TStringHelper.CompareTo(const AValue: string): Integer;
begin
  Result := CompareTo(Self, AValue);
end;

class function TStringHelper.CompareTo(const ALeft, ARight: string): Integer;
begin
  Result := AnsiCompareText(ALeft, ARight);
end;

function TStringHelper.Contains(const AValue: string): Boolean;
begin
  Result := System.Pos(AValue, Self) > 0;
end;

function TStringHelper.Equals(const AValue: string): Boolean;
begin
  Result := Self = AValue;
end;

function TStringHelper.FirstChar: Char;
begin
  if NotIsEmpty then
    Result := Chars[0]
  else
    Result := #0;
end;

function TStringHelper.Format(const AArgs: array of const): string;
begin
  Result := System.SysUtils.Format(Self, AArgs);
end;

class function TStringHelper.Format(const AValue: string; const AArgs: array of const): string;
begin
  Result := System.SysUtils.Format(AValue, AArgs);
end;

function TStringHelper.get_Chars(const AIndex: Integer): Char;
begin
  Result := Self[AIndex + 1];
end;

function TStringHelper.get_Length: Integer;
begin
  Result := System.Length(Self);
end;

function TStringHelper.High: Integer;
begin
  Result := Length - 1;
end;

function TStringHelper.IndexOf(const AValue: Char): Integer;
begin
  Result := Pos(AValue, Self, 1) - 1;
end;

function TStringHelper.IndexOf(const AValue: string; const AStartIndex, ACount: Integer): Integer;
begin
  Result := System.Pos(AValue, Self, AStartIndex + 1) - 1;
  if (Result + AValue.Length) > (AStartIndex + ACount) then
    Result := -1;
end;

function TStringHelper.IndexOf(const AValue: string): Integer;
begin
  Result := System.Pos(AValue, Self) - 1;
end;

function TStringHelper.Insert(const AStartIndex: Integer; const AValue: string): string;
begin
  System.Insert(AValue, Self, AStartIndex + 1);
  Result := Self;
end;

function TStringHelper.IsEmpty: Boolean;
begin
  Result := Self = Empty;
end;

function TStringHelper.IsNumber: Boolean;
const
  cNumbers = ['0'..'9'];
var
  lCount: Integer;
begin
  if IsEmpty then
    Exit(False);

  for lCount := 0 to Length - 1 do
  begin
    if not CharInSet(Chars[lCount], cNumbers) then
      Exit(False);
  end;
  Result := True;
end;

class function TStringHelper.Low: Integer;
begin
  Result := 0;
end;

function TStringHelper.NotEquals(const AValue: string): Boolean;
begin
  Result := not Equals(AValue);
end;

function TStringHelper.NotIsEmpty: Boolean;
begin
  Result := Self <> Empty;
end;

function TStringHelper.PadLeft(const ATotalWidth: Integer): string;
const
  cDefPadChar = ' ';
begin
  Result := PadLeft(ATotalWidth, cDefPadChar);
end;

function TStringHelper.PadLeft(ATotalWidth: Integer; const APaddingChar: Char): string;
begin
  ATotalWidth := ATotalWidth - Length;
  if ATotalWidth > 0 then
    Result := System.StringOfChar(APaddingChar, ATotalWidth) + Self
  else
    Result := Self;
end;

function TStringHelper.Remove(const AStartIndex: Integer): string;
begin
  Result := Self;
  System.Delete(Result, AStartIndex + 1, Result.Length);
end;

function TStringHelper.Remove(const AStartIndex, ACount: Integer): string;
begin
  Result := Self;
  System.Delete(Result, AStartIndex + 1, ACount);
end;

function TStringHelper.Replace(const AOldValue, ANewValue: string): string;
begin
  Result := System.SysUtils.StringReplace(Self, AOldValue, ANewValue, [rfReplaceAll]);
end;

function TStringHelper.Same(const AValue: string): Boolean;
begin
  Result := System.SysUtils.AnsiSameText(Self, AValue);
end;

procedure TStringHelper.set_Chars(const AIndex: Integer; const AValue: Char);
begin
  Self[AIndex + 1] := AValue;
end;

procedure TStringHelper.set_Length(const AValue: Integer);
begin
  System.SetLength(Self, AValue);
end;

function TStringHelper.Substring(const AStartIndex: Integer): string;
begin
  Result := System.Copy(Self, AStartIndex + 1, Self.Length);
end;

function TStringHelper.Substring(const AStartIndex, ALength: Integer): string;
begin
  Result := System.Copy(Self, AStartIndex + 1, ALength);
end;

function TStringHelper.ToLower: string;
begin
  Result := AnsiLowerCase(Self);
end;

function TStringHelper.ToUpper: string;
begin
  Result := AnsiUpperCase(Self);
end;

function TStringHelper.Trim: string;
begin
  Result := System.SysUtils.Trim(Self);
end;

{ TStringStreamHelper }

procedure TStringStreamHelper.LoadFromFile(const AFileName: string; const ALoaded: TProc);
begin
  LoadFromFile(AFileName, ALoaded, nil);
end;

procedure TStringStreamHelper.LoadFromFile(const AFileName: string; const ALoaded: TProc; const AError: TProcString);
begin
  try
    LoadFromFile(AFileName);
    if Assigned(ALoaded) then
      ALoaded;
  except
    on E: Exception do
    begin
      if Assigned(AError) then
        AError(E.Message)
      else
        raise;
    end;
  end;
end;

{ TDateTimeHelper }

function TDateTimeHelper.Equals(const AValue: TDateTime): Boolean;
begin
  Result := SameValue(Self, AValue);
end;

function TDateTimeHelper.NotEquals(const AValue: TDateTime): Boolean;
begin
  Result := not SameValue(Self, AValue);
end;

function TDateTimeHelper.ToString: string;
begin
  Result := DateTimeToStr(Self, FormatSettings);
end;

{ TCaptionHelper }

function TCaptionHelper.CompareTo(const AValue: string): Integer;
begin
  Result := CompareTo(Self, AValue);
end;

class function TCaptionHelper.CompareTo(const ALeft, ARight: string): Integer;
begin
  Result := AnsiCompareText(ALeft, ARight);
end;

function TCaptionHelper.Equals(const AValue: string): Boolean;
begin
  Result := Self = AValue;
end;

function TCaptionHelper.FirstChar: Char;
begin
  if NotIsEmpty then
    Result := Chars[0]
  else
    Result := #0;
end;

function TCaptionHelper.Format(const AArgs: array of const): string;
begin
  Result := System.SysUtils.Format(Self, AArgs);
end;

class function TCaptionHelper.Format(const AValue: string; const AArgs: array of const): string;
begin
  Result := System.SysUtils.Format(AValue, AArgs);
end;

function TCaptionHelper.get_Chars(const AIndex: Integer): Char;
begin
  Result := Self[AIndex + 1];
end;

function TCaptionHelper.get_Length: Integer;
begin
  Result := System.Length(Self);
end;

function TCaptionHelper.High: Integer;
begin
  Result := Length - 1;
end;

function TCaptionHelper.IndexOf(const AValue: Char): Integer;
begin
  Result := Pos(AValue, Self, 1) - 1;
end;

function TCaptionHelper.IndexOf(const AValue: string; const AStartIndex, ACount: Integer): Integer;
begin
  Result := System.Pos(AValue, Self, AStartIndex + 1) - 1;
  if (Result + AValue.Length) > (AStartIndex + ACount) then
    Result := -1;
end;

function TCaptionHelper.IsEmpty: Boolean;
begin
  Result := Self = Empty;
end;

function TCaptionHelper.IsNumber: Boolean;
const
  cNumbers = ['0'..'9'];
var
  lCount: Integer;
begin
  if IsEmpty then
    Exit(False);

  for lCount := 0 to Length - 1 do
  begin
    if not CharInSet(Chars[lCount], cNumbers) then
      Exit(False);
  end;
  Result := True;
end;

class function TCaptionHelper.Low: Integer;
begin
  Result := 0;
end;

function TCaptionHelper.NotEquals(const AValue: string): Boolean;
begin
  Result := not Equals(AValue);
end;

function TCaptionHelper.NotIsEmpty: Boolean;
begin
  Result := Self <> Empty;
end;

function TCaptionHelper.PadLeft(const ATotalWidth: Integer): string;
const
  cDefPadChar = ' ';
begin
  Result := PadLeft(ATotalWidth, cDefPadChar);
end;

function TCaptionHelper.PadLeft(ATotalWidth: Integer; const APaddingChar: Char): string;
begin
  ATotalWidth := ATotalWidth - Length;
  if ATotalWidth > 0 then
    Result := System.StringOfChar(APaddingChar, ATotalWidth) + Self
  else
    Result := Self;
end;

function TCaptionHelper.Remove(const AStartIndex: Integer): string;
begin
  Result := Self;
  System.Delete(Result, AStartIndex + 1, Result.Length);
end;

function TCaptionHelper.Remove(const AStartIndex, ACount: Integer): string;
begin
  Result := Self;
  System.Delete(Result, AStartIndex + 1, ACount);
end;

procedure TCaptionHelper.set_Chars(const AIndex: Integer; const AValue: Char);
begin
  Self[AIndex + 1] := AValue;
end;

procedure TCaptionHelper.set_Length(const AValue: Integer);
begin
  System.SetLength(Self, AValue);
end;

function TCaptionHelper.Substring(const AStartIndex: Integer): string;
begin
  Result := System.Copy(Self, AStartIndex + 1, Self.Length);
end;

function TCaptionHelper.Substring(const AStartIndex, ALength: Integer): string;
begin
  Result := System.Copy(Self, AStartIndex + 1, ALength);
end;

function TCaptionHelper.ToLower: string;
begin
  Result := AnsiLowerCase(Self);
end;

function TCaptionHelper.ToUpper: string;
begin
  Result := AnsiUpperCase(Self);
end;

initialization
  FormatSettings := System.SysUtils.FormatSettings;

end.
