{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit libthreejs;

interface

uses
  JS, Web, JSDelphiSystem;

const
  ThreeJsDoubleSide: NativeInt = 2;
  ThreeJsFaceColors: NativeInt = 1;
  ThreeJsVertexColors: NativeInt = 2;
  ThreeJsFlatShading: NativeInt = 1;
  ThreeJsRepeatWrapping: NativeInt = $1E3;
  clNone = -1;

type
  TColor = type NativeInt;

  TThreeJsVector3 = class (TJSObject)
  private
    fX : Double;
    fY : Double;
    fZ : Double;
  public
    constructor New; overload;
    constructor New(x : Double; y : Double; z : Double); overload;
    procedure doSet(x : Double; y : Double; z : Double);
    procedure copy(aTarget: TThreeJsVector3);
    function subVectors(a: TThreeJsVector3; b: TThreeJsVector3): TThreeJsVector3;
    function normalize: TThreeJsVector3;
    function distanceTo(aTarget: TThreeJsVector3): Double;
    property x: Double read fX write fX;
    property y: Double read fY write fY;
    property z: Double read fZ write fZ;
  end;

  TThreeJsVector2 = class (TJSObject)
  private
    fX : Double;
    fY : Double;
  public
    constructor New; overload;
    constructor New(x : Double; y : Double); overload;
    procedure doSet(x : Double; y : Double);
    property x: Double read fX write fX;
    property y: Double read fY write fY;
  end;

  TThreeJsShape = class (TJSObject)
  public
    constructor New;
    procedure moveTo(x, y: double);
    procedure lineTo(x, y: double);
  end;

  TThreeJsEuler = class (TJSObject)
  private
    fX : Double;
    fY : Double;
    fZ : Double;
  public
    property x: Double read fX write fX;
    property y: Double read fY write fY;
    property z: Double read fZ write fZ;
  end;

  TJSHTMLImageElement = class (TJSHTMLElement);

  TThreeJsTexture = class (TJSObject)
  private
    fWrapS: integer;
    fWrapT: integer;
    fRepeat: TThreeJsVector2;
    fNeedsUpdate: boolean;
  public
    constructor New(anImage: TJSHTMLImageElement);
    property wrapS: integer read fWrapS write fWrapS;
    property wrapT: integer read fWrapT write fWrapT;
    property repeats: TThreeJsVector2 read fRepeat write fRepeat;
    property needsUpdate: boolean read fNeedsUpdate write fNeedsUpdate;
  end;

  {$IFNDEF LCLTMSWEB}
  TThreeJsTextureLoadEvent = reference to procedure;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TThreeJsTextureLoadEvent = procedure of object;
  {$ENDIF}

  TThreeJsTextureLoader = class (TJSObject)
  public
    function load(aUrl: string; DoOnLoad: TThreeJsTextureLoadEvent): TThreeJsTexture;
  end;

  TThreeJsMaterial = class (TJSObject)
  private
    fSide: integer;
    fNeedsUpdate: boolean;
    fMap: TThreeJsTexture;
    fBumpMap: TThreeJsTexture;
  public
    procedure dispose;
    property side: integer read fSide write fSide;
    property needsUpdate: boolean read fNeedsUpdate write fNeedsUpdate;
    property map: TThreeJsTexture read fMap write fMap;
    property bumpMap: TThreeJsTexture read fBumpMap write fBumpMap;
  end;

  TThreeJsLineBasicMaterial = class (TThreeJsMaterial)
  private
    flineWidth: double;
  public
    constructor New(parameters: TJSObject);
    property lineWidth: double read flineWidth write flineWidth;
  end;

  TThreeJsColor = class (TJSObject)
  public
    constructor New(colorHex: JSValue);
    procedure doSet(colorHex: JSValue);
    procedure setRGB(r: double; g: double; b: double);
    // sets all 3 color components to a value Scalar (0.0 to 1.0)
    procedure setScalar(scalar: double);
    procedure setHex(aColorHex: JSValue);
    function getHex: JSValue;
    procedure setHSL(h: double; s: double; l: double);
  end;

  TThreeJsMeshLambertMaterial = class  (TThreeJsMaterial)
  private
    fEmissive: TThreeJsColor;
  public
    constructor New(parameters: TJSObject);
    property emissive: TThreeJsColor read fEmissive write fEmissive;
  end;

  TThreeJsMeshStandardMaterial = class (TThreeJsMaterial)
  public
    constructor New(parameters: TJSObject);
  end;

  TThreeJsMeshBasicMaterial = class (TThreeJsMaterial)
  public
    constructor New(parameters: TJSObject);
  end;

  TThreeJsMeshLineMaterial = class (TThreeJsMaterial)
  public
    constructor New(parameters: TJSObject);
  end;

  TThreeJsFace3 = class (TJSObject)
  private
    fVertexColors: TJSArray;
    fa: integer;
    fb: integer;
    fc: integer;
    fColor: TColor;
    fFaceIndex: integer;
    fColorHtml: TThreeJsColor;
  public
    property a: integer read fa write fa;
    property b: integer read fb write fb;
    property c: integer read fc write fc;
    property vertexColors: TJSArray read fVertexColors write fVertexColors;
    property color: TColor read fColor write fColor;
    property faceIndex: integer read fFaceIndex write fFaceIndex;
    property colorHtml: TThreeJsColor read fColorHtml write fColorHtml;
  end;

  TThreeJsBox3 = class;

  TThreeJsGeometry = class (TJSObject)
  private
    fVertices: TJSArray;
    fFaces: TJSArray;
    fColors: TJSArray;
    fBoundingbox: TThreeJsBox3;
  public
    constructor New;
    procedure computeBoundingBox;
    procedure dispose;
    property vertices: TJSArray read fVertices write fVertices;
    property faces: TJSArray read fFaces write fFaces;
    property colors: TJSArray read fColors write fColors;
    property boundingbox: TThreeJsBox3 read fBoundingbox write fBoundingbox;
  end;

  TThreeJsObject3D = class (TJSObject)
  private
    fGeometry : TThreeJsGeometry;
    fPosition : TThreeJsVector3;
    fRotation : TThreeJsEuler;
    fCastShadow : boolean;
    fReceiveShadow : boolean;
    fName: string;
    fType: string;
    fMaterial: TThreeJsMaterial;
    fLine: TThreeJsObject3D;
    fTarget: TThreeJsObject3D;
    fChildren: TJSArray;
    fCurrentHex: JSValue;
    fParent: TThreeJsObject3D;
    fVisible : boolean;
    fCenter: TThreeJsVector3;
  public
    procedure lookAt(aVector: TThreeJsVector3);
    procedure add(anObject: TThreeJsObject3D);
    function getObjectByName(aname: string): TThreeJsObject3D;
    procedure remove(anObject: TThreeJsObject3D);
    property position: TThreeJsVector3 read fPosition write fposition;
    property rotation: TThreeJsEuler read fRotation;
    property castShadow: boolean read fCastShadow write fCastShadow;
    property receiveShadow: boolean read fReceiveShadow write fReceiveShadow;
    property name: string read fName write fName;
    // "type" is reserved work in Delphi so theType
    property theType: string read fType write fType;
    property material: TThreeJsMaterial read fMaterial;
    property line: TThreeJsObject3D read fLine;
    property target: TThreeJsObject3D read fTarget;
    property children: TJSArray read fChildren;
    property currentHex: JSValue read fCurrentHex write fCurrentHex;
    property parent: TThreeJsObject3D read fParent;
    property visible: boolean read fVisible write fVisible;
    property geometry: TThreeJsGeometry read fGeometry write fGeometry;
    property center: TThreeJsVector3 read fCenter write fCenter;
  end;

  TThreeJsBox3 = class (TJSObject)
  private
    fMin: TThreeJsVector3;
    fMax: TThreeJsVector3;
  public
    procedure setFromObject(anObhect: TThreeJsObject3D);
    function getBoundingSphere: TThreeJsObject3D;
    function getCenter(aCenter: TThreeJsVector3): TThreeJsVector3;
    property min: TThreeJsVector3 read fMin write fMin;
    property max: TThreeJsVector3 read fMax write fMax;
  end;

  TThreeJsScene = class (TThreeJsObject3D)
  public
    constructor New;
  end;

  TThreeJsCamera = class (TThreeJsObject3D)
  private
    fFov: double;
    fAspect: double;
  public
    constructor New(fov, aspect, anear, afar: double);
    procedure updateProjectionMatrix;
    property fov: double read fFov write fFov;
    property aspect: double read fAspect write fAspect;
  end;
  
  TThreeJsLight = class (TThreeJsObject3D)
  private
    fColor: TColor;
    fIntensity: double;
    fDecay: double;
    fColorHtml: TThreeJsColor;
  public
    constructor New(aColorHex: JSValue); overload;
    constructor New(aColorHex: JSValue; aIntensity: double); overload;
    property color: TColor read fColor write fColor;
    property intensity: double read fIntensity write fIntensity;
    property decay: double read fDecay write fDecay;
    property colorHtml: TThreeJsColor read fColorHtml write fColorHtml;
  end;

  TThreeJsSpotLight = class (TThreeJsLight)
  end;

  TThreeJsAmbientLight = class (TThreeJsLight)
  end;  

  TThreeJsDirectionalLight = class (TThreeJsLight)
  end;

  TThreeJsAxes = class (TThreeJsObject3D)
  public
    constructor New(length : double);
  end;

  TThreeJsGrid = class (TThreeJsObject3D)
  public
    constructor New(size: integer; divisions: integer; colorCenterLine, colorGrid: JSValue);
  end;

  TThreeJsBoxGeometry = class (TThreeJsGeometry)
  public
    constructor New(width : double; height : double; depth : double);
  end;

  TThreeJsBufferGeometry = class (TThreeJsGeometry)
  public
    constructor New;
    function fromGeometry(Geometry: TThreeJsGeometry): TThreeJsBufferGeometry;
  end;

  TThreeJsExtrudeBufferGeometry = class (TThreeJsGeometry)
  public
    constructor New(shape: TThreeJsShape; extrudeSettings: TJSObject);
  end;

  TThreeJsMesh = class (TThreeJsObject3D)
  public
    constructor New(ageometry: TJSObject; amaterial: TJSObject);
  end;

  TThreeJsWebGLShadowMap = class (TJSObject)
  private
    fEnabled: boolean;
  public
    property enabled: boolean read fEnabled write fEnabled;
  end;

  TThreeJsRenderer = class (TJSObject)
  private
    fShadowMap: TThreeJsWebGLShadowMap;
    fGammaOutput: boolean;
    fGammaFactor: double;
  public
    constructor New; overload;
    constructor New(parameters: TJSObject); overload;
    function domElement: TJSElement;
    procedure setSize(width, height: integer);
    procedure render(scene: TThreeJsScene; camera: TThreeJsCamera);
    procedure setClearColor(aColor: TThreeJsColor; alpha: double);
    function getMaxAnisotropy: Integer;
    property ShadowMap: TThreeJsWebGLShadowMap read fShadowMap write fShadowMap;
    property gammaOutput: boolean read fGammaOutput write fGammaOutput;
    property gammaFactor: double read fGammaFactor write fGammaFactor;
  end;

  TThreeJsOrbitControls = class (TJSEventTarget)
  private
    fTarget : TThreeJsVector3;
    fKeySpanSpeed: double;
    fEnabled: boolean;
    fEnableRotate: boolean;
  public
    constructor New(aCamera: TThreeJsCamera; domElement: TJSElement);
    procedure saveState;
    procedure reset;
    procedure update;
    procedure rotateLeft(angleRad: double);
    procedure rotateUp(angleRad: double);
    procedure zoomIn(deltaScale: double);
    procedure panIt(deltaX, deltaY: double);
    procedure dumpPosition;
    procedure removeMouseDown;
    procedure onMouseDownEvent(Event: TJSEvent);
    procedure removeMouseMove;
    procedure onMouseMoveEvent(Event: TJSEvent);
    procedure removeKeyDown;
    procedure onKeyDownEvent(Event: TJSEvent);
    function isRotatingOrPanning: boolean;
    property target: TThreeJsVector3 read fTarget;
    property keyPanSpeed: double read fKeySpanSpeed;
    property enabled: boolean read fEnabled write fEnabled;
    property enableRotate: boolean read fEnableRotate write fEnableRotate;
  end;

  TThreeJsPlaneGeometry = class (TThreeJsGeometry)
  private
    fVerticesNeedUpdate: boolean;
    fNormalsNeedUpdate: boolean;
    fColorsNeedUpdate: boolean;
  public
    constructor New(width : double; height : double); overload;
    constructor New(width : double; height : double; widthSegments: integer); overload;
    constructor New(width : double; height : double; widthSegments: integer; heightSegments: integer); overload;
    procedure computeFaceNormals;
    procedure computeVertexNormals;
    property verticesNeedUpdate: boolean read fVerticesNeedUpdate write fVerticesNeedUpdate;
    property normalsNeedUpdate: boolean read fNormalsNeedUpdate write fNormalsNeedUpdate;
    property colorsNeedUpdate: boolean read fColorsNeedUpdate write fColorsNeedUpdate;
  end;

  TThreeJsPlaneBufferGeometry = class (TJSObject)
  public
    constructor New(width : double; height : double; widthSegments: integer; heightSegments: integer);
  end;

  TThreeJsSphereGeometry = class (TJSObject)
  public
    constructor New(radius: double; widthSegments: integer; heightSegments: integer);
  end;

  TThreeJsArrow = class (TThreeJsObject3D)
  public
    constructor New(dir, origin: TThreeJsVector3; length: double; hexColor: JSValue);
  end;

  TThreeJsRaycaster = class (TJSObject)
  public
    constructor New;
    procedure setFromCamera(coords: TThreeJsVector2; camera: TThreeJsCamera);
    function intersectObjects(objects: TJSArray; recursive: Boolean): TJSArray;
  end;

  TThreeJsCylinderGeometry = class (TJSObject)
  public
    constructor New(radiusTop, radiusBottom, height: double; radialSegments: integer);
  end;

  TThreeJsMeshLine = class (TJSObject)
  private
    fGeometry : TThreeJsGeometry;
  public
    constructor New;
    procedure setGeometry(aGeometry: TThreeJsGeometry);
    property geometry: TThreeJsGeometry read fGeometry write fGeometry;
  end;

  TThreeJsCurve = class (TJSObject)
  public
    constructor New;
  end;

  TThreeJsCatmullRomCurve3 = class (TThreeJsCurve)
  public
    constructor New(points: TJSArray);
  end;

  TThreeJsTubeGeometry = class (TThreeJsGeometry)
  public
    constructor New(path: TThreeJsCurve; tubularSegments: Integer; radius: double; radialSegments: Integer; closed: Boolean);
  end;

  TThreeJsIntersection = class (TJSObject)
  private
    fpoint: TThreeJsVector3;
    fObject: TThreeJsObject3D; 
    fDistance: double; 
    fFace: TThreeJsFace3; 
    fFaceIndex: integer; 
  public
    property point: TThreeJsVector3 read fPoint;
    // can not use "object" as property in Delphi so theObject
    property theObject: TThreeJsObject3D read fObject;
    property distance: double read fDistance;
    property faceIndex: integer read fFaceIndex;
    property face: TThreeJsFace3 read fFace;
  end;

  {$IFNDEF LCLTMSWEB}
  TThreeJsDefaultLoadingManagerLoadEvent = reference to procedure;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TThreeJsDefaultLoadingManagerLoadEvent = procedure of object;
  {$ENDIF}

  TThreeJsDefaultLoadingManager = class(TJSObject)
  public
    class var fOnLoad: TThreeJsDefaultLoadingManagerLoadEvent;
    class property onLoad: TThreeJsDefaultLoadingManagerLoadEvent read fOnLoad write fOnLoad;
  end;


  // In case of error, JS sends error object with message
  {$IFNDEF LCLTMSWEB}
  TThreeJsLoadObjectEventJS = reference to procedure(success: boolean; anObject: TJSObject);
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TThreeJsLoadObjectEventJS = procedure(success: boolean; anObject: TJSObject) of object;
  {$ENDIF}

  //From our ThreeUtils
  function makeTextSprite(text: string; x, y, z: double; parameters: TJSObject): TThreeJsObject3D;
  function createChartGrid(options: TJSObject): TThreeJsObject3D;
  function niceScale(minValue, maxValue: double): TJSObject;
  procedure setSurfaceGradient(geometry: TThreeJsGeometry; colors: TJSArray; axis: string; reverse: boolean);
  procedure loadObjectMtl(anObjName, objFile, objPath,
               mtlFile, mtlPath, texturePath: string;
               aPosition: TThreeJsVector3;
               doneCallback: TThreeJsLoadObjectEventJS);
  procedure loadObjectGltf(anObjName, gltfFile, gltfPath: string;
               aPosition: TThreeJsVector3;
               doneCallback: TThreeJsLoadObjectEventJS);
  procedure exportGltf(aScene: TThreeJsScene; gltfFileName: string);

  // Delphi counterpart for the above
type
  TTXVerticalAlign = (txvCenter, txvTop, TxvBottom);
  TTXHorizontalAlign = (txhCenter, txhLeft, TxhRight);

  // Important: If you ever change default values, change
  // them in measureTextWidth too.
  function makeATextSprite(text: string; x, y, z: double;
    isFixed: boolean;
    isDrawTopmost: boolean;
    // alignment of whole sprite with respect to x , y, z point
    valign: TTXVerticalAlign;
    halign: TTXHorizontalAlign;
    // alignment of internal text in case of Multi-line text
    halignMultiLineText: TTXHorizontalAlign;
    textColorHex: JSValue;
    fontFace: string;
    fontHeight: double; //in webGL units
    isBold: boolean;
    isTransparentFill: boolean;
    fillColorHex: JSValue;
    borderWidth: double;
    borderColorHex: JSValue;
    margin: double = 24;
    radius: double = 6
    ): TThreeJsObject3D;

   function measureTextWidth(text: string;
    fontFace: string = 'Arial';
    fontHeight: double = 1; //in webGL units
    isBold: boolean = false;
    borderWidth: double = 0;
    margin: double = 24;
    radius: double = 6
    ): double;


   function createAChartGrid(aWidth, aHeight: double;
      numDivisionsHorizontal, numDivisionsVertical : integer;
      aGridColor: JSValue; aLineColor: JSValue;
      isThickLines:boolean=false; isTransparentFill:boolean=false): TThreeJsObject3D;

   procedure getNiceScale(minVal, maxVal: double;
                    out niceMinimum: double;
                    out niceMaximum: double;
                    out tickSpacing: double);

   procedure setASurfaceGradient(geometry: TThreeJsGeometry;
            // Just send colors and they will be used to
            // make a uniform gradient across a geometry
            // in a direction
            colors: TJSArray;
            axis: string;
            reverse: boolean);

   function makeLegendSprite(
              lines: string;
              // pass color1, color2, ... corresponding to lines
              colorList: TJSArray;
              shapeList: TJSArray;
              textColorHex: JSValue;
              fontFace: string;
              fontHeight: double; //in webGL units
              isBold: boolean;
              isTransparentFill: boolean;
              fillColorHex: JSValue;
              borderWidth: double;
              borderColorHex: JSValue;
              margin: double;
              radius: double
              ): TThreeJsObject3D;

  function getColorObject(aColorHex: JSValue): TJSObject;
  function getTColor(aThreeJsColor: TThreeJsColor): TColor;
  function ColorToHtmlNumber(aColor: TColor): JSValue;

implementation
{$HINTS OFF}
{$WARNINGS OFF}

constructor TThreeJsVector3.New;
begin
end;

constructor TThreeJsVector3.New(x : Double; y : Double; z : Double);
begin
end;

procedure TThreeJsVector3.doSet(x : Double; y : Double; z : Double);
begin
end;

procedure TThreeJsVector3.copy(aTarget: TThreeJsVector3);
begin
end;

function TThreeJsVector3.subVectors(a: TThreeJsVector3; b: TThreeJsVector3): TThreeJsVector3;
begin
  result := nil;
end;

function TThreeJsVector3.normalize: TThreeJsVector3;
begin
  result := nil;
end;

function TThreeJsVector3.distanceTo(aTarget: TThreeJsVector3): Double;
begin
  result := 0;
end;

constructor TThreeJsVector2.New;
begin
end;

constructor TThreeJsVector2.New(x : Double; y : Double);
begin
end;

procedure TThreeJsVector2.doSet(x : Double; y : Double);
begin
end;

constructor TThreeJsShape.New;
begin
end;

procedure TThreeJsShape.moveTo(x, y: double);
begin
end;

procedure TThreeJsShape.lineTo(x, y: double);
begin
end;

constructor TThreeJsTexture.New(anImage: TJSHTMLImageElement);
begin
end;

function TThreeJsTextureLoader.load(aUrl: string; DoOnLoad: TThreeJsTextureLoadEvent): TThreeJsTexture;
begin
  result := nil;
end;

procedure TThreeJsMaterial.dispose;
begin
end;

constructor TThreeJsLineBasicMaterial.New(parameters: TJSObject);
begin
end;

constructor TThreeJsColor.New(colorHex: JSValue);
begin
end;

procedure TThreeJsColor.doSet(colorHex: JSValue);
begin
end;

procedure TThreeJsColor.setRGB(r: double; g: double; b: double);
begin
end;

procedure TThreeJsColor.setScalar(scalar: double);
begin
end;

procedure TThreeJsColor.setHex(aColorHex: JSValue);
begin
end;

function TThreeJsColor.getHex: JSValue;
begin
end;

procedure TThreeJsColor.setHSL(h: double; s: double; l: double);
begin
end;

constructor TThreeJsMeshLambertMaterial.New(parameters: TJSObject);
begin
end;

constructor TThreeJsMeshStandardMaterial.New(parameters: TJSObject);
begin
end;

constructor TThreeJsMeshBasicMaterial.New(parameters: TJSObject);
begin
end;

constructor TThreeJsMeshLineMaterial.New(parameters: TJSObject);
begin
end;

constructor TThreeJsGeometry.New;
begin
end;

procedure TThreeJsGeometry.computeBoundingBox;
begin
end;

procedure TThreeJsGeometry.dispose;
begin
end;

procedure TThreeJsObject3D.lookAt(aVector: TThreeJsVector3);
begin
end;

procedure TThreeJsObject3D.add(anObject: TThreeJsObject3D);
begin
end;

function TThreeJsObject3D.getObjectByName(aname: string): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TThreeJsObject3D.remove(anObject: TThreeJsObject3D);
begin
end;

procedure TThreeJsBox3.setFromObject(anObhect: TThreeJsObject3D);
begin
end;

function TThreeJsBox3.getBoundingSphere: TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsBox3.getCenter(aCenter: TThreeJsVector3): TThreeJsVector3;
begin
  result := nil;
end;

constructor TThreeJsScene.New;
begin
end;

constructor TThreeJsCamera.New(fov, aspect, anear, afar: double);
begin
end;

procedure TThreeJsCamera.updateProjectionMatrix;
begin
end;

constructor TThreeJsLight.New(aColorHex: JSValue);
begin
end;

constructor TThreeJsLight.New(aColorHex: JSValue; aIntensity: double);
begin
end;

constructor TThreeJsAxes.New(length : double);
begin
end;

constructor TThreeJsGrid.New(size: integer; divisions: integer; colorCenterLine, colorGrid: JSValue);
begin
end;

constructor TThreeJsBoxGeometry.New(width : double; height : double; depth : double);
begin
end;

constructor TThreeJsBufferGeometry.New;
begin
end;

function TThreeJsBufferGeometry.fromGeometry(Geometry: TThreeJsGeometry): TThreeJsBufferGeometry;
begin
  result := nil;
end;

constructor TThreeJsExtrudeBufferGeometry.New(shape: TThreeJsShape; extrudeSettings: TJSObject);
begin
end;

constructor TThreeJsMesh.New(ageometry: TJSObject; amaterial: TJSObject);
begin
end;

constructor TThreeJsRenderer.New;
begin
end;

constructor TThreeJsRenderer.New(parameters: TJSObject);
begin
end;

function TThreeJsRenderer.domElement: TJSElement;
begin
  result := nil;
end;

procedure TThreeJsRenderer.setSize(width, height: integer);
begin
end;

procedure TThreeJsRenderer.render(scene: TThreeJsScene; camera: TThreeJsCamera);
begin
end;

procedure TThreeJsRenderer.setClearColor(aColor: TThreeJsColor; alpha: double);
begin
end;

function TThreeJsRenderer.getMaxAnisotropy: Integer;
begin
end;

constructor TThreeJsOrbitControls.New(aCamera: TThreeJsCamera; domElement: TJSElement);
begin
end;

procedure TThreeJsOrbitControls.saveState;
begin
end;

procedure TThreeJsOrbitControls.reset;
begin
end;

procedure TThreeJsOrbitControls.update;
begin
end;

procedure TThreeJsOrbitControls.rotateLeft(angleRad: double);
begin
end;

procedure TThreeJsOrbitControls.rotateUp(angleRad: double);
begin
end;

procedure TThreeJsOrbitControls.zoomIn(deltaScale: double);
begin
end;

procedure TThreeJsOrbitControls.panIt(deltaX, deltaY: double);
begin
end;

procedure TThreeJsOrbitControls.dumpPosition;
begin
end;

procedure TThreeJsOrbitControls.removeMouseDown;
begin
end;

procedure TThreeJsOrbitControls.onMouseDownEvent(Event: TJSEvent);
begin
end;

procedure TThreeJsOrbitControls.removeMouseMove;
begin
end;

procedure TThreeJsOrbitControls.onMouseMoveEvent(Event: TJSEvent);
begin
end;

procedure TThreeJsOrbitControls.removeKeyDown;
begin
end;

procedure TThreeJsOrbitControls.onKeyDownEvent(Event: TJSEvent);
begin
end;

function TThreeJsOrbitControls.isRotatingOrPanning: boolean;
begin
  result := false;
end;

constructor TThreeJsPlaneGeometry.New(width : double; height : double);
begin
end;

constructor TThreeJsPlaneGeometry.New(width : double; height : double; widthSegments: integer);
begin
end;

constructor TThreeJsPlaneGeometry.New(width : double; height : double; widthSegments: integer; heightSegments: integer);
begin
end;

procedure TThreeJsPlaneGeometry.computeFaceNormals;
begin
end;

procedure TThreeJsPlaneGeometry.computeVertexNormals;
begin
end;

constructor TThreeJsPlaneBufferGeometry.New(width : double; height : double; widthSegments: integer; heightSegments: integer);
begin
end;

constructor TThreeJsSphereGeometry.New(radius: double; widthSegments: integer; heightSegments: integer);
begin
end;

constructor TThreeJsArrow.New(dir, origin: TThreeJsVector3; length: double; hexColor: JSValue);
begin
end;

constructor TThreeJsRaycaster.New;
begin
end;

procedure TThreeJsRaycaster.setFromCamera(coords: TThreeJsVector2; camera: TThreeJsCamera);
begin
end;

function TThreeJsRaycaster.intersectObjects(objects: TJSArray; recursive: Boolean): TJSArray;
begin
  Result := nil;
end;

constructor TThreeJsCylinderGeometry.New(radiusTop, radiusBottom, height: double; radialSegments: integer);
begin
end;

constructor TThreeJsMeshLine.New;
begin
end;

procedure TThreeJsMeshLine.setGeometry(aGeometry: TThreeJsGeometry);
begin
end;

constructor TThreeJsCurve.New;
begin
end;

constructor TThreeJsCatmullRomCurve3.New(points: TJSArray);
begin
end;

constructor TThreeJsTubeGeometry.New(path: TThreeJsCurve; tubularSegments: Integer; radius: double; radialSegments: Integer; closed: Boolean);
begin
end;

function makeTextSprite(text: string; x, y, z: double; parameters: TJSObject): TThreeJsObject3D;
begin
  result := nil;
end;

function createChartGrid(options: TJSObject): TThreeJsObject3D;
begin
  result := nil;
end;

function niceScale(minValue, maxValue: double): TJSObject;
begin
  result := nil;
end;

procedure setSurfaceGradient(geometry: TThreeJsGeometry; colors: TJSArray; axis: string; reverse: boolean);
begin
end;

function makeATextSprite(text: string; x, y, z: double;
    isFixed: boolean;
    isDrawTopmost: boolean;
    // alignment of whole sprite with respect to x , y, z point
    valign: TTXVerticalAlign;
    halign: TTXHorizontalAlign;
    // alignment of internal text in case of Multi-line text
    halignMultiLineText: TTXHorizontalAlign;
    textColorHex: JSValue;
    fontFace: string;
    fontHeight: double; //in webGL units
    isBold: boolean;
    isTransparentFill: boolean;
    fillColorHex: JSValue;
    borderWidth: double;
    borderColorHex: JSValue;
    margin: double = 24;
    radius: double = 6
    ): TThreeJsObject3D;
begin
  result := nil;
end;

function measureTextWidth(text: string;
    fontFace: string = 'Arial';
    fontHeight: double = 1; //in webGL units
    isBold: boolean = false;
    borderWidth: double = 0;
    margin: double = 24;
    radius: double = 6
    ): double;
begin
  result := 0;
end;

function createAChartGrid(aWidth, aHeight: double;
      numDivisionsHorizontal, numDivisionsVertical : integer;
      aGridColor: JSValue; aLineColor: JSValue;
      isThickLines:boolean=false; isTransparentFill:boolean=false): TThreeJsObject3D;
begin
  result := nil;
end;

procedure getNiceScale(minVal, maxVal: double;
                    out niceMinimum: double;
                    out niceMaximum: double;
                    out tickSpacing: double);
begin
end;

function makeLegendSprite(
              lines: string;
              // pass color1, color2, ... corresponding to lines
              colorList: TJSArray;
              shapeList: TJSArray;
              textColorHex: JSValue;
              fontFace: string;
              fontHeight: double; //in webGL units
              isBold: boolean;
              isTransparentFill: boolean;
              fillColorHex: JSValue;
              borderWidth: double;
              borderColorHex: JSValue;
              margin: double;
              radius: double
              ): TThreeJsObject3D;
begin
  result := nil;
end;

function getColorObject(aColorHex: JSValue): TJSObject;
begin
  result := nil;
end;

procedure setASurfaceGradient(geometry: TThreeJsGeometry;
            // Just send colors and they will be used to
            // make a uniform gradient across a geometry
            // in a direction
            colors: TJSArray;
            axis: string;
            reverse: boolean);
begin
end;

procedure loadObjectMtl(anObjName, objFile, objPath,
               mtlFile, mtlPath, texturePath: string;
               aPosition: TThreeJsVector3;
               doneCallback: TThreeJsLoadObjectEventJS);
begin
end;

procedure loadObjectGltf(anObjName, gltfFile, gltfPath: string;
               aPosition: TThreeJsVector3;
               doneCallback: TThreeJsLoadObjectEventJS);
begin
end;

procedure exportGltf(aScene: TThreeJsScene; gltfFileName: string);
begin
end;

function getTColor(aThreeJsColor: TThreeJsColor): TColor;
begin
  result := clNone;
end;

function ColorToHtmlNumber(aColor: TColor): JSValue;
begin
  result := 0;
end;

end.
